%This file solves the incomplete markets model. 

%Started on Feb 3, 2016, this one has non-separable preferences as in the Shimer (AEJ) paper:
%u(c,l) = (c^(1-sigma) (1 + (sigma - 1)*phi * l^(1+gamma)/(1+gamma))^sigma - 1)/(1-sigma) 
%we ignore the constant, of course, but notice that with sigma = 1, this
%boils down to our separable case.

%This one has a two-dimensional shock process, started on 06/26/2014.

%FOR THE LONG-RUN INEQUALITY PROJECT, set equal taue = taus and set qe = .2

%This one has 1 type of capital, 2 types of endogeneous labor and depreciation.
%Shocks are shocks to labor productivity, efficient labor = exp(z)*ell.
%Shocks are within occupational group.

%This version has non-linear income taxes and thus needs to solve a
%non-linear equation in c and l.
 
%It uses the endogenous grid method and does everything in matrix form for any number of shocks.

%Inner loop:
%Fixes labor policies.
%Uses matlab to find the equilibrium prices, transfers, B and calibrates
%omega, nu, mu as well as betas and betau to match the approprite targets.
%It sets G as as a fraction of output (gy = 16% in benchmark).

%Outer loop:
%Find labor policies consistent with consumption.

%clc
clear all
warning off all
%profile on

tic;
global nA A a m alpha eta omega rho nu k0 ds de lu ls rs re tauke tauks ks ke taul nY sigma Ys Yu open v phi betas delta  piuentr pisentr ...
    lambdas lambdau lambdasa lambdaua gy x pis piu Ts Tu Amat Tlambdau Tlambdas tauc by lambdal lspol lupol lsinv luinv  cspol cupol qe qs TFP...
    NFAP varRoW rhoRoW TFProw gdp betarow betau sizebeta pibetas pibetau

display 'This version has entrants draw from a different that stationary distribution and only works for 3 shocks!'
%In Hong et al, in fact the entrants draw from a very similar distribution as the stationary one.

TFP=1;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
open = 1; %set to 1 for open economy
calibrate = 0; %calibrating betaRoW in the benchmark 2000 simulation, check file RoWbetatfp.m
normalize = 1;
rhoRoW = 0.9335;
varRoW = 0.0158; %this is the persistent component 


sigma = 2; %CRRA in consumption
gamma = NaN; %power disutility of labor

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
delta = 1- 0.022; %probability of retiring in Castaneda, D-J, R-R (2003)

%U(c,l) = c^(1-sigma)/(1-sigma)*v(l); %where l is labor
%need a value for phi for the very first iteration, cannot solve the EGM without it

shimer = 1;

if shimer ==1
gamma = 2; %power disutility of labor
%this the Shimer specification
v  = @(phi, el) (1 + (sigma - 1)*phi * el.^(1+gamma)/(1+gamma)).^sigma; 
vp = @(phi, el) sigma*(1 + (sigma - 1)*phi*el.^(1+gamma)/(1+gamma)).^(sigma-1)*(sigma - 1)*phi.*el.^gamma; 
else
%this the Cobb-Douglas specification used in Krueger-Ludwig and elsewhere
% note that gamma is not used, phi should be in (0,1)
 v  = @(phi, el) phi*(1-el).^((1/phi-1)*(1-sigma)); 
 vp = @(phi, el) -((1/phi-1)*(1-sigma))*phi*(1-el).^((1/phi-1)*(1-sigma)-1);
end

load input input
phi = input(1);
omega = input(2);
nu = input(3);
betas = input(4:5);
betau = input(7:8);
sizebeta =length(betas);

pibetas = [0.5, 0.5];
pibetau = [0.5, 0.5];

 if open == 1;    
 betarow = input(10)
% %betarow = 0.918592024073999
 TFProw = input(11)
 varRoW = input(12);
 end
input;

%technology
eta = .401; %KORV
rho = -.495;%KORV
alpha = .117; %KORV
%alpha = input(6);
de = .124;%GHK
ds = .056;%GHK
 

%TRADITIONAL CHANNELS
qe = 1; %qe = .1870; %from cummins-violante for 01-05 using avg growth over 67-00 interpolation
%qe = .1577; %from FRED for 2010
%qe = .0746; %from FRED for 2016
%qe = .1076; %from FRED average 2007-2016
%qe = .1489; %from FRED average 2001-2010

qs = 1;

%total measure of agents of each type calibrated from CPS
pis = .1356; %in 1967
%pis = .3454; %in 2016
%pis = .3544; %in 2017
%pis = .3169; % in 2010
piu = 1-pis;

%GVT POLICIES
gy = 0.16; %share of G in Y
 %government debt as share of gdp held by the public net of foreigners in 1967 (this is fact the average in 1970, since we do not earlier data)

%auerbach tax differentials, 1960-1969, %gravelle tax differentials, 2010,  %progressivity tax parameter in Ferriere, in 2010
tauks = 1-.85*(1-.49); tauke = 1-.85*(1-.41) ;taul = 0.124; by = 0.25;
%tauks = 1-.85*(1-.32); tauke = 1-.85*(1-.26); taul = 0.095; by = 0.32;
if open == 1
    by = 0.26;
%    by = 0.60;
end

%taul = .181;
tauc = 0;

% %RISK
% %Hong et al, see Table 2 and Table 5
% %quite different from HSV (if we average those out) and also from Krueger-Ludwig
% %UPDATED 2016-09, FROM WHAT THEY SENT US
  persu = 0.9859; perss = 0.9834; varsentr = .1172; varuentr = .1488;
%  varomegas = 0.0037; varomegau = 0.0052; varnus = 0.0116; varnuu = 0.0177; %1967 average of the first 5
%  varomegas = 0.0304;  varomegau = 0.0157;   varnus = 0.0673;   varnuu = 0.0627; %2010 average the last 3
% varomegas = 0.0089; varomegau = 0.0073; varnus = 0.0201; varnuu = 0.0259; %1967 average of the first 10 (without NA), 1967-1976
 varomegas = 0.0229; varomegau = 0.0146; varnus = 0.0916; varnuu = 0.0739; %2010 average of the last 10 (without NA), 2001 - 2010 
 
%varnus = 0.0673;   varnuu = 0.0627;
% varnus = 0.0201; varnuu = 0.0259;
  
% %based on HSV, see HSVdata.xls
% pers = 0.9733; perss = pers; persu = pers;
% %varomega = 0.0093; varomegas = varomega; varomegau = varomega; %67-71
% varomega = 0.0200; varomegas = varomega; varomegau = varomega; %96-00
% %varnu = 0.0314; varnus = varnu; varnuu = varnu; %in 67-71
% varnu = 0.0746; varnus = varnu; varnuu = varnu; %in 96-00
% varentr = 0.1242; varsentr = varentr; varuentr = varentr;

amin = 0;
amax = 500;

%A = linspace(amin, amax, 201)'
%controlling the density of grid points close to 0
%A = ((exp(linspace(0,log(exp(2)*amax+1), 101)) - 1)/exp(2))'; %for 2-by-2 shocks
%A = ((exp(linspace(0,log(exp(2)*amax+1), 51)) - 1)/exp(2))';%- 0.036
A = ((exp(linspace(0,log(exp(4)*amax+1), 51)) - 1)/exp(4))' - 0.01236;
nA=length(A);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%discretizing the shock processes by rouwenhorst
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
nY = 3;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%skilled people
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

rouw = rouwenhorst(perss, varomegas^.5, nY); Yeta = (rouw(1,:))'; Teta = rouw(2:nY+1,:);
rouw = rouwenhorst(0, varnus^.5, nY); Ynu = (rouw(1,:))'; Tnu = rouw(2:nY+1,:);

 options.disp = 0;
 %[vector value] = eigs(Teta',1,1.1, options);
 %pietaSS = vector(:,1)/sum(vector(:,1))'
 [vector value] = eigs(Tnu',1,1.1, options);
 pinuSS = vector(:,1)/sum(vector(:,1))';
% meanexpeta = exp(Yeta')*pietaSS;
% meanexpnu = exp(Ynu')*pinuSS;
% cdfeta = cumsum(pietaSS); cdfnu = cumsum(pinuSS); Tetacdf = cumsum(Teta,2); Tnucdf = cumsum(Tnu,2);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%ENTRANTS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
pentr = 1- varsentr/Yeta(1)^2;
pietaentr = [(1-pentr)/2; pentr; (1-pentr)/2];
pisentr = kron(pietaentr,pinuSS);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%constructing the two-dimensional wage vector
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Ys = (exp(kron(Yeta, ones(length(Ynu),1))).*exp(kron(ones(length(Yeta),1),Ynu)));Ts = kron(Teta,Tnu);

%constructing the 'true' transition matrix which takes death into acount
Ttrues = delta*Ts+ (1-delta)*kron(pisentr', ones(nY^2,1));

%normalizing
%options.disp = 0;
[vector value] = eigs(Ttrues',1,1.1, options);
pitruesSS = vector(:,1)/sum(vector(:,1))';
meanYs = Ys'*pitruesSS;
%(meanexpeta*meanexpnu) - meanYs
if normalize == 1
Ys = Ys/meanYs; 
end
Ys'*pitruesSS;

%cdfs = cumsum(pisSS); Tscdf = cumsum(Ts,2);
% %check properties of the processes approximated by rouwenhorst
% sim = 0;
% if sim ==1;
%     display 'Properties of the wage process for the skilled'
%     simulation(cdfeta, Yeta, Tetacdf, cdfnu, Ynu, Tnucdf, cdfs, Ys, Tscdf, nY); 
% end

pdfeta1 = pietaentr';
for i=1:50
    i;
pdfetai=pdfeta1*Teta^i;
cdfetai = cumsum(pdfetai);
Tetacdf = cumsum(Teta,2);
checks(i,:) = simulationeta(cdfetai, Yeta, 0, 0, Yeta, Tetacdf, nY); 


%function simulationeta = simulationeta(cdfeta1, Yeta1, Tetacdf1, cdfeta2, Yeta2, Tetacdf2, nY) 
end


Ys = Ys';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%unskilled people
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rouw = rouwenhorst(persu, varomegau^.5, nY); Yeta = (rouw(1,:))'; Teta = rouw(2:nY+1,:);
rouw = rouwenhorst(0, varnuu^.5, nY); Ynu = (rouw(1,:))'; Tnu = rouw(2:nY+1,:);

options.disp = 0;
%[vector value] = eigs(Teta',1,1.1, options);
%pietaSS = vector(:,1)/sum(vector(:,1))'
[vector value] = eigs(Tnu',1,1.1, options);
pinuSS = vector(:,1)/sum(vector(:,1))';
% meanexpeta = exp(Yeta')*pietaSS;
% meanexpnu = exp(Ynu')*pinuSS;
% cdfeta = cumsum(pietaSS); cdfnu = cumsum(pinuSS); Tetacdf = cumsum(Teta,2); Tnucdf = cumsum(Tnu,2);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%ENTRANTS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 pentr = 1 - varuentr/Yeta(1)^2;
 pietaentr = [(1-pentr)/2; pentr; (1-pentr)/2];
 piuentr = kron(pietaentr,pinuSS);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%constructing the two-dimensional wage vector
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Yu = (exp(kron(Yeta, ones(length(Ynu),1))).*exp(kron(ones(length(Yeta),1),Ynu))); Tu = kron(Teta,Tnu);

%constructing the 'true' transition matrix which takes death into acount
Ttrueu = delta*Tu+ (1-delta)*kron(piuentr', ones(nY^2,1));

%normalizing
%options.disp = 0;
[vector value] = eigs(Ttrueu',1,1.1, options);
pitrueuSS = vector(:,1)/sum(vector(:,1))';
meanYu = Yu'*pitrueuSS;
%(meanexpeta*meanexpnu) - meanYs
if normalize == 1
Yu = Yu/meanYu;
end
Yu'*pitrueuSS;

% cdfu = cumsum(piuSS); Tucdf = cumsum(Tu,2);
% %check properties of the processes approximated by rouwenhorst
% sim = 0;
% if sim ==1;
%      display 'Properties of the wage process for the unskilled'
%     simulation(cdfeta, Yeta, Tetacdf, cdfnu, Ynu, Tnucdf, cdfu, Yu, Tucdf, nY); 
% end

pdfeta1 = pietaentr';
for i=1:50
    i;
pdfetai=pdfeta1*Teta^i;
cdfetai = cumsum(pdfetai);
Tetacdf = cumsum(Teta,2);
checku(i,:) = simulationeta(cdfetai, Yeta, 0, 0, Yeta, Tetacdf, nY); 


%function simulationeta = simulationeta(cdfeta1, Yeta1, Tetacdf1, cdfeta2, Yeta2, Tetacdf2, nY) 
end

Yu = Yu';

%clearvars cdfeta Yeta Tetacdf cdfnu Ynu Tnucdf cdfu Tucdf cdfs Tscdf meanYu meanYs vector value pinuSS pietaSS
nY = nY^2;
Amat = kron(A, ones(1,nY));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%'steady state' wage residuals, ignoring age, because the means of log w
%are the same within both types, scaling (sp) does not matter
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
logresYs = log(Ys) - log(Ys)*pitruesSS;
logresYu = log(Yu) - log(Yu)*pitrueuSS;

%finding the varlog of residual wages
wresprob = [[logresYs';logresYu'],[pis*pitruesSS;piu*pitrueuSS]];
meanreslogw = wresprob(:,1)'*wresprob(:,2);
varlogresw = wresprob(:,2)'*(wresprob(:,1)-meanreslogw).^2;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%initial guesses
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%loading prices etc.
loading = 1;
if loading == 1;
    load z z
    z0 = z;
    
    %z0
    
    load lspol lspol
    load lupol lupol
    load lsinv lsinv
    load luinv luinv
    
%      lspol = 1/3*ones(nA, nY);
%      lupol = 1/3*ones(nA, nY);
%      lsinv = lspol;
%      luinv = lupol;
%          lspol(:,9) = 1/3*ones(nA, 1);
%          lsinv(:,9) = 1/3*ones(nA, 1);
else %if not a good initial guess available
    display 'You need a good initial guess'
    bla
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%loop in labor income to get consistency between exogeneous and endogeneous labor
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
diffl = 1;
xl = 1;
norml = 10^-6;

while (diffl > norml) & (xl <50) %iteration on labor income
    
    %aiyagarisimEE_v3(z0)
    %bla
    
tic; %to measure the time
options = optimset('display', 'iter', 'tolfun', 10^(-10), 'tolX', 10^(-10), 'maxfunevals', 1000000, 'maxiter', 100); 
%options = optimset('display', 'iter', 'tolfun', 10^(-10), 'tolX', 10^(-10), 'maxfunevals', 1000000, 'maxiter', 20, 'NonlEqnAlgorithm', 'gn'); 
%options = optimset('display', 'iter', 'tolfun', 10^(-10), 'tolX', 10^(-10), 'maxfunevals', 1000000, 'maxiter', 20, 'NonlEqnAlgorithm', 'lm'); 
%options = optimset('display', 'iter', 'tolfun', 10^(-10), 'tolX', 10^(-10), 'maxfunevals', 1000000, 'maxiter', 20, 'largescale', 'on'); 


%this function solves the 'inner loop' finds prices and calibrates parameters holding labor policies fixed
z = fsolve(@aiyagarisimEE_v3, z0, options);
z0 = z;
aiyagarisimEE_v3(z); %to get the right aprime policies
%z = z0;
toc;

load asprime
load auprime

%for invertability at the borrowing constraint
for b = 1:sizebeta
for ii = 1:nY
    for jj = 2:20
        if auprime(jj,ii,b) == min(A);
            auprime(jj,ii,b) = min(A) + jj*10^-10;
        end
        if asprime(jj,ii,b) == min(A);
            asprime(jj,ii,b) = min(A) + jj*10^-10;
        end
    end
end
end


%defining variables
rs = z(1);
r = rs;
%re = (1-tauks)*(rs - ds)/(1-tauke) + de;%the NAC
w = z(2:3);
lambdal = z(4);
B = z(5);
meany = z(6);



saving = 1;
if saving == 1;%this saves the current combination that goes through
    
    save z z
   % save phi phi
   % save asprime asprime
   % save auprime auprime
    save lspol lspol
    save lupol lupol
    save lsinv lsinv
    save luinv luinv
    save lambdas lambdas
    save lambdau lambdau
end

%returns
re = (1-tauks)/(1-tauke)*qe/qs*(rs - qs*ds) + qe*de;% %the NAC %AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
R = 1+(rs-qs*ds)*(1-tauks)/qs; % AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA

for b = 1:sizebeta
%stationary distribution
lambdasmat(:,:,b) = (reshape(lambdas(:,b), nY, nA))';
lambdaumat(:,:,b) = (reshape(lambdau(:,b), nY, nA))';

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%constructing labor policies that would be consistent with consumption
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%OPTION FIXED POINT
%define intra FOC
%sigma = 2;

%MAYBE SHOULD DO IT IN A LOOP?
if sigma == 1; %this is the separable utility case we plug in for v
    display 'This only works for sigma <> 1'
    bla
end
 options = optimset('display', 'off', 'tolfun', 10^(-10), 'tolX', 10^(-10), 'maxfunevals', 1000000, 'maxiter', 1000); 
 %Gauss-Newton works well for Shimer
 %options = optimset('display', 'off', 'tolfun', 10^(-10), 'tolX', 10^(-10), 'maxfunevals', 1000000, 'maxiter', 1000, 'NonlEqnAlgorithm', 'gn'); 
 %options = optimset('display', 'off', 'tolfun', 10^(-10), 'tolX', 10^(-10), 'maxfunevals', 1000000, 'maxiter', 1000, 'Algorithm', 'levenberg-marquardt'); 
 %options = optimset('display', 'iter', 'tolfun', 10^(-10), 'tolX', 10^(-10), 'maxfunevals', 1000000, 'maxiter', 1000, 'largescale', 'on');
 
  lspol0 = lspol;
  lupol0 = lupol;
 for b = 1:sizebeta
     for k = 1:nA
        for m = 1:nY
           intras = @(xs) - vp(phi, (atan(xs) + pi/2)/pi)./v(phi, (atan(xs) + pi/2)/pi) - ((1-sigma)*lambdal*(Ys(m)*w(2).*(atan(xs) + pi/2)/pi/meany).^(-taul).*(Ys(m)*w(2))*(1-taul))./((1+tauc)*cspol(k,m,b));
           intrau = @(xu) - vp(phi, (atan(xu) + pi/2)/pi)./v(phi, (atan(xu) + pi/2)/pi) - ((1-sigma)*lambdal*(Yu(m)*w(1).*(atan(xu) + pi/2)/pi/meany).^(-taul).*(Yu(m)*w(1))*(1-taul))./((1+tauc)*cupol(k,m,b));
           xsnew(k,m,b) = fsolve(@(xs) intras(xs), tan(lspol0(k,m,b)*pi - pi/2), options);
           xunew(k,m,b) = fsolve(@(xu) intrau(xu), tan(lupol0(k,m,b)*pi - pi/2), options);
           
        end
    end
 
 end
  
 lspolnew = (atan(xsnew) + pi/2)/pi;
 lupolnew = (atan(xunew) + pi/2)/pi;



diffl = sum(sum(sum(lambdasmat.*abs(lspolnew - lspol) + lambdaumat.*abs(lupolnew - lupol))))

%diffl = 0;

%diffl = diffl + diffphiiter

if diffl > norml
    %setting the polices for next round
    polcoeff = 0.5;
    lspol = polcoeff*lspolnew+(1-polcoeff)*lspol;
    lupol = polcoeff*lupolnew+(1-polcoeff)*lupol;
    
   %Constructing l(a') to be consistent with both the new l(a,z) and a'(a,z)
    for b = 1:sizebeta
    for k = 1:nA
        for m = 1:nY
           lsinvnew(k,m,b) = interp1(asprime(:, m, b), lspol(:,m, b), A(k),'linear','extrap'); %not ignoring the BC, but can have only 1 guy on it
           luinvnew(k,m,b) = interp1(auprime(:, m, b), lupol(:,m, b), A(k),'linear','extrap'); %not ignoring the BC, but can have only 1 guy on it
           
           %linv(k,m) = interp1(aprimeraw(:, m), lpol(:,m), A(k),'linear','extrap'); %ignoring the BC
           %linv(k,m) = interp1(aprime(:, m), lpol(:,m), A(k), 'linear','extrap'); 
           %using linear interpolation in Matlab, doesn't work with binding BC
           %linv(k,m) = interp1(A, lpol(:,m), ainv(k,m), 'linear', 'extrap'); %alternative mapping between l(a,z) and a'(a,z)
        end
    end
    end
%difflinv = sum(sum(lambdasmat.*abs(lsinvnew - lsinv))) + sum(sum(lambdaumat.*abs(luinvnew - luinv)));

%setting the polices for next round
lsinv = lsinvnew;
luinv = luinvnew;
% %this was needed in the transition codes:
% lsinv = max(lsinvnew, 10^-9);
% luinv = max(luinvnew, 10^-9);

xl = xl + 1


end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%computing stuff once everything has converged
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%aggregate effective labors
ls = sum(sum(sum(kron(Ys, ones(nA,1)).*lambdasmat.*lspol)));
lu = sum(sum(sum(kron(Yu, ones(nA,1)).*lambdaumat.*lupol)));

%average labor
l1 = sum(sum(sum(lambdasmat.*lspol)))/pis;
l2 = sum(sum(sum(lambdaumat.*lupol)))/piu;
lagg = pis*l1+piu*l2;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%checking feasibility and a few other things
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%cdf

% if trim == 1 %deleting the lowest wage draw of the unskilled
%     lambdaumat(:,1) = lambdaumat(:,1)*0   
%     aggcdf = sum(sum(lambdaumat + lambdasmat))
%     lambdaumat = lambdaumat/aggcdf;
%     lambdasmat = lambdasmat/aggcdf;
%     lambdasa = sum(lambdasmat,2);
%     lambdaua = sum(lambdaumat,2);
% end


%lambdasmatcdf = cumsum(lambdasmat,1);
%lambdaumatcdf = cumsum(lambdaumat,1);
lambdasacdf = cumsum(lambdasa);
lambdauacdf = cumsum(lambdaua);
%average consumptions
c1 = sum(sum(sum(lambdasmat.*cspol)))/pis;
c2 = sum(sum(sum(lambdaumat.*cupol)))/piu;
cagg = sum(sum(sum(lambdasmat.*cspol)) + sum(sum(lambdaumat.*cupol)));%aggregate consumption
iagg = de*ke + ds*ks; %aggregate investment

% brack = (omega*ke^rho + (1-omega)*ls^rho);
% parent = ((1-nu)*lu^eta + nu*brack^(eta/rho));
% gdp = ks^alpha* parent^((1-alpha)/eta)
gdp = ycomp(x); %compute output using ycomp.m
G = gy*gdp; %government exepnditure
gdpnet = gdp - qs*ds*ks - qe*de*ke;  %AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
check1 = gdp - lu*w(1) - ls*w(2) - rs*ks - re*ke
check2 = gdp-cagg - qe*de*ke - qs*ds*ks - G  %AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA

MP = ycompprime2(x); %computing the marginal products of factors using ycompprime2.m
w = [MP(3), MP(4)]; 

sp = (Ys*pitruesSS*w(2))/(Yu*pitrueuSS*w(1)); %whether or not the mean is 1 in both sectors - should weight by hours?
KtoY = (qe*ke+qs*ks)/gdp; %AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
lshare = (lu*w(1) + ls*w(2))/gdp; %labor share

NFAP = lambdasa'*A + lambdaua'*A - B - (qs*ks+qe*ke); %net foreign asset position should be 0 for a closed economy
%domestic asset holdings
assets = lambdasa'*A + lambdaua'*A;



%pre tax earnings
ears = kron(Ys.*w(2),ones(nA,1)).*lspol;%for each agent
earu = kron(Yu.*w(1),ones(nA,1)).*lupol;%for each agent
aears = sum(sum(lambdasmat.*ears))/pis;%average earnings for skilled
aearu = sum(sum(lambdaumat.*earu))/piu;%average earnings for unskilled
%aear = aears*pis + aearu*piu; %should be equal to meany

for b = 1:sizebeta
    
    %pre tax earnings
    ears(:,:,b) = kron(Ys.*w(2),ones(nA,1)).*lspol(:,:,b);%for each agent
    earu(:,:,b) = kron(Yu.*w(1),ones(nA,1)).*lupol(:,:,b);%for each agent
    
    %defining after tax labor income to be used below
    Ysmat(:,:,b) = lambdal*(kron(Ys*w(2), ones(nA,1)).*lspol(:,:,b)/meany).^(1-taul)*meany;
    Yumat(:,:,b) = lambdal*(kron(Yu*w(1), ones(nA,1)).*lupol(:,:,b)/meany).^(1-taul)*meany;
    
    %total pre-tax income 
    incomes(:,:,b) = ears(:,:,b) + (rs-ds)*Amat;
    incomeu(:,:,b) = earu(:,:,b) + (rs-ds)*Amat;
    
end

aears = sum(sum(sum(lambdasmat.*ears)))/pis;%average earnings for skilled
aearu = sum(sum(sum(lambdaumat.*earu)))/piu;%average earnings for unskilled

%aftertax earnings
earsat = Ysmat;
earuat = Yumat;
%average taxes
ats  = (ears-Ysmat)./ears;
atu  = (earu-Yumat)./earu;
ats = sum(sum(sum(lambdasmat.*ats)))/pis;
atu = sum(sum(sum(lambdaumat.*atu)))/piu;
atagg = ats*pis + atu*piu;



%tax revenues
ltax = sum(sum(sum((ears-Ysmat).*lambdasmat +(earu-Yumat).*lambdaumat)));

ctax = (MP(1) - qs*ds)*ks*tauks +  (MP(2) - qe*de)*ke*tauke; 
corptax = ((1-(1-tauks)/.85)*(MP(1) - qs*ds)*ks + (1-(1-tauke)/.85)*(MP(2) - qe*de)*ke); 
%tax rates faced by the firms for consistency
taufe = 1-(1-tauke)/.85;
taufs = 1-(1-tauks)/.85;
cctax = ((1-taufs)*(MP(1) - qs*ds)*ks + (1-taufe)*(MP(2) - qe*de)*ke)*.15;

%compute relative capital holdings - how do KL (2013) do on this?
rks = lambdasa'*A/pis/(lambdaua'*A/piu);



%finding the asset gini
 lambdaa = lambdaua + lambdasa;
 asset = A.*lambdaa;
 si = cumsum(asset,1);
 siminus = [0; si(1:nA-1)];
 Giniasset = 1 - (lambdaa'*(si+siminus))/si(nA);
 
   %variance of log assets
 meanasset = log(A(2:length(A)))'*lambdaa(2:length(A));
 varlogasset = ((log(A(2:length(A)))-meanasset).^2)'*lambdaa(2:length(A));
 
% % %finding the asset quintiles
  lambdaacdf = lambdauacdf + lambdasacdf;
%  quint = lambdaacdf*ones(1,4) - kron([.5,.9,.95,.99], ones(nA,1));
%  quint = quint < 0;
%  %(A*ones(1,4)).*quint.*(lambdaa*ones(1,4));
%  quintiles = [sum((A*ones(1,4)).*quint.*(lambdaa*ones(1,4)))/si(nA),1];
%  quintiles = quintiles - [0, quintiles(1:4)];
 
  
 %computing asset quintiles (i.e. share of total asset holdings for the
 %whole quintile)
 mat = [lambdaacdf, A];
 %pickquint = [.2,.4,.6,.8];
 pickquint = [.5,.9,.95,.99];
  %pickquint = [.6, .9];
 for nn = 1:length(pickquint);
    for ii = 1:nA
        if  lambdaacdf(ii) > pickquint(nn);
            mat(nA+nn,:) = [pickquint(nn), A(ii)];
        break
        end
    end
 end
 matsort = sortrows(mat,1);
 
  
 templambdaa = matsort(:,1) - [0;matsort(1:size(matsort,1)-1,1)];
 quint = matsort(:,1)*ones(1,length(pickquint)) - kron(pickquint, ones(nA+length(pickquint),1));
 quint = quint <= 0;
 quintiles = [sum(( matsort(:,2)*ones(1,length(pickquint))).*quint.*(templambdaa*ones(1,length(pickquint))))/si(nA),1];
 assetquintiles = quintiles - [0, quintiles(1:length(pickquint))];
 
 %computing asset percentiles now, i.e. the amount of assets amn agent at a
 %particular place in the asset distribution holds
 %very similar to before
 pickperc = [.1,.25,.50,.75, .90, .95, .99];
 for nn = 1:length(pickperc);
    for ii = 1:nA
        if  lambdaacdf(ii) > pickperc(nn);
            assetperc(nn) = A(ii);
        break
        end
    end
 end
 assetperc;
 
 %finding the hours varlog and gini
 elprob = [[reshape(lspol, nA*nY*sizebeta,1);reshape(lupol, nA*nY*sizebeta,1)], [reshape(lambdasmat,nA*nY*sizebeta,1); reshape(lambdaumat, nA*nY*sizebeta,1)]];
 %variance of log hours
 meanel = log(elprob(:,1))'*elprob(:,2);
 varlogel = ((log(elprob(:,1))-meanel).^2)'*elprob(:,2);
 elprobsort = sortrows(elprob,1);
 eltot = elprobsort(:,1).*elprobsort(:,2);
 si = cumsum(eltot,1);
 probcdf = cumsum(elprobsort(:,2));
 %plot(probcdf, si);
 siminus = [0; si(1:2*nY*nA*sizebeta-1)];
 Giniel = 1 - (elprobsort(:,2)'*(si+siminus))/si(2*nY*nA*sizebeta);
 
 
%   figure(6)
%   plot(elprobsort(:,1), probcdf, 'r');
%   title('labor cdf')


% %finding the pre-tax labor earnings varlog and gini
 earprob = [[reshape(ears, nA*nY*sizebeta,1);reshape(earu, nA*nY*sizebeta,1)],  [reshape(lambdasmat,nA*nY*sizebeta,1); reshape(lambdaumat, nA*nY*sizebeta,1)]];
 %variance of log earnings
 meanear = log(earprob(:,1))'*earprob(:,2);
 varlogear = ((log(earprob(:,1))-meanear).^2)'*earprob(:,2);
 earprobsort = sortrows(earprob,1);
 earning = earprobsort(:,1).*earprobsort(:,2);
 si = cumsum(earning,1);
 probcdf = cumsum(earprobsort(:,2));
 %plot(probcdf, si);
 siminus = [0; si(1:2*nY*nA*sizebeta-1)];
 Gini = 1 - (earprobsort(:,2)'*(si+siminus))/si(2*nY*nA*sizebeta);



 
 
 %computing log consumption variance 
 consprob = [[reshape(cspol, nA*nY*sizebeta,1);reshape(cupol, nA*nY*sizebeta,1)],  [reshape(lambdasmat,nA*nY*sizebeta,1); reshape(lambdaumat, nA*nY*sizebeta,1)]];
 meancons = log(consprob(:,1))'*consprob(:,2);
 varlogcons = ((log(consprob(:,1))-meancons).^2)'*consprob(:,2);
 
 
 
  %for the pass-through calculation
  for b = 1:sizebeta
    for jj = 1:nA
        
        for ii = 1:nY

         for ll = 1:nY

             kk = (jj-1)*nY + ii;
        consstplus1(kk, ll,b) = interp1(A, cspol(:,ll,b), asprime(jj, ii, b),'linear','extrap');
        earnstplus1(kk, ll,b) = interp1(A, ears(:,ll, b), asprime(jj, ii, b),'linear','extrap');
        probstplus1(kk,ll,b) = lambdas(kk,b)* Ts(ii,ll);
        consutplus1(kk, ll,b) = interp1(A, cupol(:,ll, b), auprime(jj, ii, b),'linear','extrap');
        earnutplus1(kk, ll,b) = interp1(A, earu(:,ll, b), asprime(jj, ii),'linear','extrap');
        probutplus1(kk,ll,b) = lambdau(kk,b)* Tu(ii,ll);
         end
     end
    end
 conss(:,:,b) = kron(reshape(cspol(:,:,b)', nA*nY,1), ones(1,nY));
 consu(:,:,b) = kron(reshape(cupol(:,:,b)', nA*nY,1), ones(1,nY));
 earns(:,:,b) = kron(reshape(ears(:,:,b)', nA*nY,1), ones(1,nY));
 earnu(:,:,b) = kron(reshape(earu(:,:,b)', nA*nY,1), ones(1,nY));
  end
  
  

 
 cons = [conss;consu];
 earn = [earns;earnu];
 %corr(reshape(cons,2*nA*nY*nY,1), reshape(earn,2*nA*nY*nY,1))
 constplus1 = [consstplus1; consutplus1];
 earntplus1 = [earnstplus1; earnutplus1];
 %corr(reshape(constplus1,2*nA*nY*nY,1), reshape(earntplus1,2*nA*nY*nY,1))
 probtplus1 = [probstplus1; probutplus1];
 sum(sum(probtplus1));
 
 regx = log(earntplus1) - log(earn);
 regy = log(constplus1) - log(cons);
 %corr(reshape(regx,2*nA*nY*nY,1), reshape(regy,2*nA*nY*nY,1))
 
 meanregx = sum(sum(sum(regx.*probtplus1)));
 meanregy = sum(sum(sum(regy.*probtplus1)));
 varx =  sum(sum(sum((regx-meanregx).^2.*probtplus1)));
 covar = sum(sum(sum(regy.*regx.*probtplus1))) - meanregx*meanregy;
 passthrough = covar/varx
 

 
  earprobs = [reshape(ears, nA*nY*sizebeta,1), reshape(lambdasmat,nA*nY*sizebeta,1)];
  earprobu = [reshape(earu, nA*nY*sizebeta,1), reshape(lambdaumat, nA*nY*sizebeta,1)];
  
 %finding the within group earnings varlog and gini 
% earprobs = [reshape(ears', nA*nY,1), lambdas/pis];
% earprobu = [reshape(earu', nA*nY,1), lambdau/piu];
 
  earprobs = [reshape(ears, nA*nY*sizebeta,1), reshape(lambdasmat,nA*nY*sizebeta,1)/pis];
  earprobu = [reshape(earu, nA*nY*sizebeta,1), reshape(lambdaumat,nA*nY*sizebeta,1)/piu];
 
 %variance of log earnings
 meanears = log(earprobs(:,1))'*earprobs(:,2);
 meanearu = log(earprobu(:,1))'*earprobu(:,2);
 varlogears = ((log(earprobs(:,1))-meanears).^2)'*earprobs(:,2);
 varlogearu = ((log(earprobu(:,1))-meanearu).^2)'*earprobu(:,2);
 
 earprobsorts = sortrows(earprobs,1);
 earnings = earprobsorts(:,1).*earprobsorts(:,2);
 sis = cumsum(earnings,1);
 probcdfs = cumsum(earprobsorts(:,2));
 siminuss = [0; sis(1:nY*nA*sizebeta-1)];
 Ginis = 1 - (earprobsorts(:,2)'*(sis+siminuss))/sis(nY*nA*sizebeta);
 
 earprobsortu = sortrows(earprobu,1);
 earningu = earprobsortu(:,1).*earprobsortu(:,2);
 siu = cumsum(earningu,1);
 probcdfu = cumsum(earprobsortu(:,2));
 siminusu = [0; siu(1:nY*nA*sizebeta-1)];
 Giniu = 1 - (earprobsortu(:,2)'*(siu+siminusu))/siu(nY*nA*sizebeta);
 
% %finding the earnings quintiles SHOULD TAKE INTO ACCOUNT DISCRETE JUMPS
%  quintear = probcdf*ones(1,4) - kron([.5, .9,.95,.99], ones(nA*nY*2,1));
%  quintear = quintear < 0;
%  quintilesear = [sum((earprobsort(:,1)*ones(1,4)).*quintear.*(earprobsort(:,2)*ones(1,4)))/si(nA*nY*2),1];
%  quintilesear = quintilesear - [0, quintilesear(1:4)];
%  %90-10 ratio
%  ninetyten = max(quintear(:,4).*earprobsort(:,1))/max(quintear(:,1).*earprobsort(:,1));
 
 %computing earnings quintiles (i.e. share of total asset holdings for the
 %whole quintile)
 mat = [cumsum(earprobsort(:,2)), earprobsort(:,1)];
 sizemat = size(mat,1);
 %pickquint = [.2,.4,.6,.8];
 pickquint = [.5,.9,.95,.99];
 for nn = 1:length(pickquint);
    for ii = 1:size(mat,1)
        if  mat(ii,1) > pickquint(nn);
            mat(sizemat+nn,:) = [pickquint(nn), mat(ii,2)];
        break
        end
    end
 end
 mat;
 matsort = sortrows(mat,1);
 
  
 templambdaear = matsort(:,1) - [0;matsort(1:size(matsort,1)-1,1)];
 quint = matsort(:,1)*ones(1,length(pickquint)) - kron(pickquint, ones(sizemat+length(pickquint),1));
 quint = quint <= 0;
 quintiles = [sum(( matsort(:,2)*ones(1,length(pickquint))).*quint.*(templambdaear*ones(1,length(pickquint))))/si(sizemat),1];
 earquintiles = quintiles - [0, quintiles(1:length(pickquint))];
 
 %computing asset percentiles now, i.e. the amount of assets amn agent at a
 %particular place in the asset distribution holds
 %very similar to before
  pickperc = [.1,.25,.50,.75, .90, .95, .99];
  for nn = 1:length(pickperc);
    for ii = 1:size(mat,1)
        if  mat(ii,1) > pickperc(nn);
           earperc(nn) = mat(ii,2);
        break
        end
    end
 end
 earperc; 
 
%finding the wage gini
 wprob = [[w(2)*Ys';w(1)*Yu'],[pis*pitruesSS;piu*pitrueuSS]];
 wprobsort = sortrows(wprob,1);
 wage = wprobsort(:,1).*wprobsort(:,2);
 si = cumsum(wage,1);
 probcdf = cumsum(wprobsort(:,2));
 %plot(probcdf, si);
 siminus = [0; si(1:2*nY-1)];
 Giniw = 1 - (wprobsort(:,2)'*(si+siminus))/si(2*nY);
%finding the varlog
logw = log(wprob(:,1));
meanlogw = logw'*wprob(:,2);
varlogw = wprob(:,2)'*(logw-meanlogw).^2;


   pickperc = [.1,.25,.50,.75, .90, .95, .99];
  for nn = 1:length(pickperc);
    for ii = 1:size(wprobsort,1)
        if  probcdf(ii,1) >= pickperc(nn);
           wageperc(nn) = wprobsort(ii,1);
        break
        end
    end
 end
 wageperc;

 
 % elprob = [[reshape(lspol, nA*nY*sizebeta,1);reshape(lupol, nA*nY*sizebeta,1)], [reshape(lambdasmat,nA*nY*sizebeta,1); reshape(lambdaumat, nA*nY*sizebeta,1)]];
 
 for b = 1:sizebeta
    wagemats(:,:,b) = kron(ones(nA,1),w(2)*Ys);
    wagematu(:,:,b) = kron(ones(nA,1),w(1)*Yu);
 end
 
 %finding the wage hours correlation
wagefull =[reshape(wagemats, nA*nY*sizebeta,1);reshape(wagematu, nA*nY*sizebeta,1)];
matcorr = [log(wagefull),log(elprob(:,1)), elprob(:,2)];

meanlogwage = matcorr(:,1)'*matcorr(:,3);
stdlogwage = (matcorr(:,3)'*(matcorr(:,1)-meanlogwage).^2)^.5;
meanlogel = matcorr(:,2)'*matcorr(:,3);
stdlogel = (matcorr(:,3)'*(matcorr(:,2)-meanlogel).^2)^.5;
corrwageel =  matcorr(:,3)'*((matcorr(:,1)-meanlogwage).*(matcorr(:,2)-meanlogel));


%variance of the peristent component
TT = 35;
vars(1) = varsentr;
varu(1) = varuentr;
for tt = 1:TT
    
    vars(tt+1) = perss^2*vars(tt) + varomegas;
    varu(tt+1) = persu^2*varu(tt) + varomegau;
end

vars(TT+1);
varu(TT+1);
40*varomegas;
40*varomegau;

%this model does not generate enough wealth and in particular earnings
 %dispersion, so the welfare gains are probably a lower bound
 %including capital income makes the gini smaller, probably because the
 %correlation is not positive (rich people work less)

 
 
  %finding the unskilled asset gini
 asset = A.*lambdaua/piu;
 si = cumsum(asset,1);
 siminus = [0; si(1:nA-1)];
 Giniau = 1 - (lambdaua'*(si+siminus))/si(nA)/piu;
 
 
  %finding the skilled asset gini
 asset = A.*lambdasa/pis;
 si = cumsum(asset,1);
 siminus = [0; si(1:nA-1)];
 Ginias = 1 - (lambdasa'*(si+siminus))/si(nA)/pis;
 
 
 
 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %recover the values
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

tic;
display 'now computing the value functions'
 if sigma == 1
    display 'This does not work for sigma = 1 yet.'
 else
    Vs = zeros(nA, nY, sizebeta);
    Vu = zeros(nA, nY, sizebeta);    
    
 load Vs Vs
 load Vu Vu    
     
 Vsalt = Vs;
 Vualt = Vu;  
 
    diffv = 10^10;
    xv = 0;
    normv = 10^-7;

while diffv > normv
for b = 1:sizebeta
    for k = 1:nA
        for m = 1:nY
            
%                TVs(k,m) = cspol(k,m)^(1-sigma)/(1-sigma)*v(phi, lspol(k,m)) + delta*betas*Ts(m,:)*(interp1(A, Vs, asprime(k,m),'linear','extrap'))';
%                TVu(k,m) = cupol(k,m)^(1-sigma)/(1-sigma)*v(phi, lupol(k,m)) + delta*betau*Tu(m,:)*(interp1(A, Vu, auprime(k,m),'linear','extrap'))';
                
           for bb = 1:nY
               Vsprime(bb,b) = bfendo2(asprime(k,m,b), nA, 1, A)*Vsalt(:,bb,b);
               Vuprime(bb,b) = bfendo2(auprime(k,m,b), nA, 1, A)*Vualt(:,bb,b);
           end
           
           TVsalt(k,m,b) = cspol(k,m,b)^(1-sigma)/(1-sigma)*v(phi, lspol(k,m,b)) + delta*betas(b)*Ts(m,:)*Vsprime(:,b); 
           TVualt(k,m,b) = cupol(k,m,b)^(1-sigma)/(1-sigma)*v(phi, lupol(k,m,b)) + delta*betau(b)*Tu(m,:)*Vuprime(:,b);

         end
    end
end
%     diffv = sum(sum(abs(Vs-TVs))) + sum(sum(abs(Vu-TVu)))
%     Vs = TVs;
%     Vu = TVu;
    
    diffv = sum(sum(sum(abs(Vsalt-TVsalt)+ abs(Vualt-TVualt))));
    Vsalt = TVsalt;
    Vualt = TVualt;
    
    xv = xv+1;
end
 end
 
display 'computing the value functions takes'

%to speed this us: (i) do in matrix form, (ii) use matrix inversion, 
%(iii) use equality of derivatives
toc;

Vs = Vsalt;
Vu = Vualt;

save Vs Vs
save Vu Vu


 Vmeans = sum(sum(sum(Vs.*lambdasmat)))/pis;
 Vmeanu = sum(sum(sum(Vu.*lambdaumat)))/piu;

 Vmean = pis*Vmeans + piu*Vmeanu;

 %Vszero = Vs(1,:,:)*pisentr;
 %Vuzero = Vu(1,:)*piuentr;
 Vszero = NaN;
 Vuzero = NaN;
 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%approximating the cost function with exponential dsitribution
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%lambdaexp = -log(piu)/(Vszero - Vuzero);
lambdaexp = NaN;

output = [pis, qs, qe, sigma, gamma, delta, phi, alpha, omega, nu,  betas, betau, lambdal, meany, G, B, Vszero, Vuzero,  Vszero - Vuzero, Vmeans, Vmeanu, Vmean, lambdaexp, 1/lambdaexp, tauks, tauke, ctax, corptax, ...
    cctax, taul, ltax, ats atu, tauc, gdp, G/gdp, B/gdp, qs*ks, qe*ke,  qe*ke+qs*ks, qe*ke/(qe*ke+qs*ks), (qe*ke+qs*ks)/gdp, assets,  assets - B, NFAP/gdp, NFAP/(qe*ke+qs*ks), NFAP/(assets-B), ...
     cagg, iagg, iagg/gdp, iagg/(qe*ke+qs*ks), c1, c2, lagg, l1, l2, ls, lu, w(1), w(2), sp, (lu*w(1) + ls*w(2))/gdp, rks, rs, re, (rs-qs*ds)*(1-tauks)/qs, (re-qe*de)*(1-tauke)/qe ...
      (rs-qs*ds), (re-qe*de), ((rs-qs*ds)*ks/qs + (re-qe*de)*ke/qe)/(ks+ke),...
      varlogel, Giniel, corrwageel,...
        varlogresw, varsentr, varomegas/(1-perss^2), vars(TT+1), varuentr, varomegau/(1-persu^2), varu(TT+1)...
      varlogw, Giniw, wageperc, wageperc(3)/wageperc(1), wageperc(5)/wageperc(3), ...
      varlogear,  Gini, varlogears,  Ginis, varlogearu, Giniu, earquintiles, earperc, earperc(3)/earperc(1), earperc(5)/earperc(3), Giniasset, Ginias, Giniau, assetquintiles, assetquintiles(5) + assetquintiles(4) + assetquintiles(3), ...
      assetperc,  0 , 0, 0 , varlogcons, varlogcons/varlogear, passthrough]';

%save output output
%profile off

display 'Fraction of people with negative assets:'
sum((A<0).*lambdaa)
display 'Fraction of people with approx non-negative assets:'
sum((A<0.002).*lambdaa)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%open economy calibration
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if open ==1;
  display 'OPEN ECONOMY IS NOT ALL THAT OPEN HERE'

  
if calibrate == 1

options = optimset('display', 'iter', 'tolfun', 10^(-10), 'tolX', 10^(-10), 'maxfunevals', 1000000, 'maxiter', 100); 
% %options = optimset('display', 'iter', 'tolfun', 10^(-10), 'tolX', 10^(-10), 'maxfunevals', 1000000, 'maxiter', 20, 'NonlEqnAlgorithm', 'gn'); 
% %options = optimset('display', 'iter', 'tolfun', 10^(-10), 'tolX', 10^(-10), 'maxfunevals', 1000000, 'maxiter', 20, 'NonlEqnAlgorithm', 'lm'); 
% %options = optimset('display', 'iter', 'tolfun', 10^(-10), 'tolX', 10^(-10), 'maxfunevals', 1000000, 'maxiter', 20, 'largescale', 'on'); 
betarownew = fsolve(@RoWbetatfp, betarow, options);
betarow = betarownew;
end

output = [pis, qs, qe, sigma, gamma, delta, phi, alpha, omega, nu,  betas, betau, lambdal, meany, G, B, Vszero, Vuzero,  Vszero - Vuzero, Vmeans, Vmeanu, Vmean, lambdaexp, 1/lambdaexp, tauks, tauke, ctax, corptax, ...
    cctax, taul, ltax, ats atu, tauc, gdp, G/gdp, B/gdp, qs*ks, qe*ke,  qe*ke+qs*ks, qe*ke/(qe*ke+qs*ks), (qe*ke+qs*ks)/gdp, assets, assets - B, NFAP/gdp, NFAP/(qe*ke+qs*ks), NFAP/(assets-B), ...
     cagg, iagg, iagg/gdp, iagg/(qe*ke+qs*ks), c1, c2, lagg, l1, l2, ls, lu, w(1), w(2), sp, (lu*w(1) + ls*w(2))/gdp, rks, rs, re, (rs-qs*ds)*(1-tauks)/qs, (re-qe*de)*(1-tauke)/qe ...
    (rs-qs*ds), (re-qe*de), ((rs-qs*ds)*ks/qs + (re-qe*de)*ke/qe)/(ks+ke), ...
       varlogel, Giniel, corrwageel,...
          varlogresw, varsentr, varomegas/(1-perss^2), vars(TT+1), varuentr, varomegau/(1-persu^2), varu(TT+1)...
    varlogw, Giniw, wageperc,  wageperc(3)/wageperc(1), wageperc(5)/wageperc(3), ...
    varlogear, Gini, varlogears,  Ginis, varlogearu, Giniu, earquintiles, earperc, earperc(3)/earperc(1), earperc(5)/earperc(3), Giniasset, Ginias, Giniau,  assetquintiles, assetquintiles(5) + assetquintiles(4) + assetquintiles(3), assetperc ...
    betarow, TFProw, varRoW,  varlogcons, varlogcons/varlogear, passthrough]';
end

xSS2 = [x, lambdal, gdp, 1+(rs-qs*ds)*(1-tauks)/qs, B/gdp]
save('SS2', 'xSS2', 'asprime', 'auprime', 'lspol', 'lupol', 'lsinv', 'luinv', 'lambdas', 'lambdau')


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%figures
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
figures = 0;
if figures  == 1

figure(1)
plot(A, lambdasmat, 'r');
hold on;
plot(A, lambdaumat, 'b');
title('pdfs')
% hold on;
% plot(A, lambdasa, 'r');
% hold on;
% plot(A, lambdaua, 'b');


figure(2)
plot(A, cspol, 'r');
hold on;
plot(A, cupol, 'b');
title('consumption')

 figure(3)
 plot(A, lspol, 'r');
 hold on;
 plot(A, lupol, 'b');
 title('labor')
 
figure(4)
plot(A, lambdasmatcdf, 'r');
hold on;
plot(A, lambdaumatcdf, 'b');
title('cdfs')
% 
 figure(5)
 plot(A, ears, 'r');
 hold on;
 plot(A, earu, 'b');
 title('pre-tax earnings')

%
%  figure(5)
%  plot(A, asprime, 'r');
%  hold on;
%  plot(A, auprime, 'b');
%  hold on;
%  plot(A,A, 'k');

% figure(6)
% mesh(Ys, A, Vs);
% hold on;
% mesh(Yu
%{
display 'differences for skilled across persistent shocks'
[min(lspol(:,7) - lspol(:,4)), max(lspol(:,7) - lspol(:,4))]
[min(lspol(:,4) - lspol(:,1)), max(lspol(:,4) - lspol(:,1))]
[min(lspol(:,8) - lspol(:,5)), max(lspol(:,8) - lspol(:,5))]
[min(lspol(:,5) - lspol(:,2)), max(lspol(:,5) - lspol(:,2))]
[min(lspol(:,9) - lspol(:,6)), max(lspol(:,9) - lspol(:,6))]
[min(lspol(:,6) - lspol(:,3)), max(lspol(:,6) - lspol(:,3))]

display 'differences for skilled across transitory shocks'
[min(lspol(:,3) - lspol(:,2)), max(lspol(:,3) - lspol(:,2))]
[min(lspol(:,2) - lspol(:,1)), max(lspol(:,2) - lspol(:,1))]
[min(lspol(:,6) - lspol(:,5)), max(lspol(:,6) - lspol(:,5))]
[min(lspol(:,5) - lspol(:,4)), max(lspol(:,5) - lspol(:,4))]
[min(lspol(:,9) - lspol(:,8)), max(lspol(:,9) - lspol(:,8))]
[min(lspol(:,8) - lspol(:,7)), max(lspol(:,8) - lspol(:,7))]

display 'differences for skilled across persistent shocks'
[min(lupol(:,7) - lupol(:,4)), max(lupol(:,7) - lupol(:,4))]
[min(lupol(:,4) - lupol(:,1)), max(lupol(:,4) - lupol(:,1))]
[min(lupol(:,8) - lupol(:,5)), max(lupol(:,8) - lupol(:,5))]
[min(lupol(:,5) - lupol(:,2)), max(lupol(:,5) - lupol(:,2))]
[min(lupol(:,9) - lupol(:,6)), max(lupol(:,9) - lupol(:,6))]
[min(lupol(:,6) - lupol(:,3)), max(lupol(:,6) - lupol(:,3))]

display 'differences for skilled across transitory shocks'
[min(lupol(:,3) - lupol(:,2)), max(lupol(:,3) - lupol(:,2))]
[min(lupol(:,2) - lupol(:,1)), max(lupol(:,2) - lupol(:,1))]
[min(lupol(:,6) - lupol(:,5)), max(lupol(:,6) - lupol(:,5))]
[min(lupol(:,5) - lupol(:,4)), max(lupol(:,5) - lupol(:,4))]
[min(lupol(:,9) - lupol(:,8)), max(lupol(:,9) - lupol(:,8))]
[min(lupol(:,8) - lupol(:,7)), max(lupol(:,8) - lupol(:,7))]
%}
end